#include "ioC8051F320.h"
#include "USB_API.h"
#define USBXpress_int 0x83

BYTE pac[64];
BYTE SPIbuffer[256];

void SPI_Init(void);

__code const unsigned int USB_VID = 0x10C4;
__code const unsigned int USB_PID = 0xEA61;
__code const char USB_MaxPower = 15; // maximaler Strom 2x15 mA
__code const unsigned int USB_bcdDevice = 0x0100; // Release nummer 1.0
__code const char USB_PwAttributes = 0x80; // Bus-Powered, no remote wakeup
__code const BYTE USB_SerialStr[]  = {0x0A, 0x03, 'f', 0, 'i', 0, 'n', 0, 'e', 0};
__code const BYTE USB_ProductStr[] = {0x10, 0x03, 'U', 0, 'S', 0, 'B', 0, ' ', 0, 'A', 0, 'P', 0, 'I', 0};
__code const BYTE USB_MfrStr[] = {0x1A, 0x03, 'S', 0, 'i', 0, 'l', 0, 'i', 0, 'c', 0, 'o', 0, 'n', 0,  ' ', 0, 'L', 0, 'a', 0, 'b', 0, 's', 0};

unsigned int count = 0;
unsigned int bufferSize = 4096;

void main(void) {
  P1MDIN = 0xBF; // P1.6 analog input Poti
  P2MDOUT = 0x99; // P2.0 LED, P2.3 Reset_n, P2.7 WakeUp | 2.4 blaue LED
  P1MDOUT = 0x08; // Taster fr BaseInfo
  P1SKIP = 0x40; // Poti
  SPI_Init();
  XBR0 = 0x02;
  XBR1 = 0x40;
  
  OSCICN |= 0x03; // 12 MHz
  
  USB_Clock_Start();
  USB_Init(USB_VID, USB_PID, (BYTE *)USB_MfrStr, (BYTE *)USB_ProductStr, (BYTE *)USB_SerialStr, USB_MaxPower, USB_PwAttributes, USB_bcdDevice);
  USB_Int_Enable();

  IE |= 0x80; asm("NOP");asm("NOP");asm("NOP");asm("NOP");
  IE |= 0x40; // SPI

  while(1) {
  }
}

#pragma vector=SPIF_int
__interrupt void SPI_ISR(void) {
  SPI0CN &= ~0xF0; // alle Flags lschen

  SPIbuffer[count++] = SPI0DAT;
  SPIbuffer[count] = 0;
}

void SPI_Init(void) {
  SPI0CFG &= ~0x30;
  SPI0CFG |= 0x10; // CPHA = 1 CPOL = 0
  SPI0CN = 0x00; // This setting should be done before enabling SPI on crossbar when 3-wire mode
  P0MDIN = 0x0F;
  SPI0CN |= 0x01;
}

#pragma vector=USBXpress_int
__interrupt void USB_API_TEST_ISR(void) {
  char INTVAL = Get_Interrupt_Source();

  if (INTVAL & RX_COMPLETE) {
    USB_Int_Disable();
    
    Block_Read(pac, 7);
    bufferSize = pac[0];// | (pac[2] << 8);
    while (!(P0 & 0x08)); // warten bis NSS high ist
    IE &= ~0x40; // SPI Interrupt ausschalten - alles was ab hier noch kommt, geht verloren!!!
    Block_Write(SPIbuffer, bufferSize);
    count = 0;
    while (!(P0 & 0x08)); // falls NSS low, warten bis es wieder high ist - Paket geht verloren
    IE |= 0x40; // SPI Interrupt wieder an

    if (!pac[1]) { // Programmstart auf dem PC // Reset der Basisstation - NEU: Start der base()-Methode auf dem Knoten. Reset frht zu eimen Bitbertragungsfehler!
      IE &= ~0x40; // SPI
      P1 ^= 0x08;
      asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");      asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");      asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");      asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");      asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");      asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");      asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");      asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");asm("NOP");
      P1 ^= 0x08;
      IE |= 0x40; // SPI
    }
    
    USB_Int_Enable();
  }
  if (INTVAL & TX_COMPLETE) {
  }
  if (INTVAL & DEV_SUSPEND){
    P0MDIN = 0x00;
    P1MDIN = 0x00;
    P2MDIN = 0x00;
    P3MDIN = 0x00;
    USB_Suspend();
    P1MDIN = 0xBF; // P1.6 analog input Poti
    P2MDOUT = 0x99; // P2.0 LED, P2.3 Reset_n, P2.7 WakeUp | 2.4 blaue LED
    P1SKIP = 0x40; // Poti
  }
  if (INTVAL & DEV_CONFIGURED){
    P1MDIN = 0xBF; // P1.6 analog input Poti
    P2MDOUT = 0x99; // P2.0 LED, P2.3 Reset_n, P2.7 WakeUp | 2.4 blaue LED
    P1SKIP = 0x40; // Poti
  }
}

/*#pragma vector=TF2H_int
__interrupt void Timer2High_ISR(void) {
  if (TMR2CN & 0x80) {
    TMR2CN &= ~0xC0;
    P2 ^= 0x01; // LED zeigt bertragungsfehler an
  }
  if (TMR2CN & 0x40) {
    TMR2CN &= ~0x40;
  }
}*/